package ru.amse.javadependencies.zhukova.modelbuilding;


import ru.amse.javadependencies.zhukova.data.IClass;
import ru.amse.javadependencies.zhukova.data.IPackage;
import ru.amse.javadependencies.zhukova.data.dependencies.IClassDependency;
import ru.amse.javadependencies.zhukova.data.dependencies.IDependencyGraph;
import ru.amse.javadependencies.zhukova.model.*;
import ru.amse.javadependencies.zhukova.model.impl.*;
import ru.amse.javadependencies.zhukova.ui.graphpane.GraphPane;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.awt.*;


public class GraphModelBuilder implements IGraphModelBuilder {
    private final IDependencyGraph<IClass> myClassDependencyGraph;
    private final IDependencyGraph<IPackage> myPackageDependencyGraph;

    public GraphModelBuilder(IDependencyGraph<IClass> classDependencyGraph, IDependencyGraph<IPackage> packageDependencyGraph) {
        myClassDependencyGraph = classDependencyGraph;
        myPackageDependencyGraph = packageDependencyGraph;
    }

    public GraphModel buildGraphModel(GraphPane graphPane) {
        GraphModel gr = new GraphModel(graphPane);
        Map<String, SuperVertex> packageNameToVertex = new HashMap<String, SuperVertex>();
        Map<String, SimpleVertex> classNameToVertex = new HashMap<String, SimpleVertex>();
        buildVertices(graphPane, gr, packageNameToVertex, classNameToVertex);
        buildArcs(classNameToVertex, gr);        
        gr.setNameToSuperVertexMap(packageNameToVertex);
        gr.setNameToSimpleVertexMap(classNameToVertex);
        return gr;
    }

    private void buildVertices(GraphPane graphPane, IGraphModel gr, Map<String, SuperVertex> packageNameToVertex, Map<String, SimpleVertex> classNameToVertex) {
        Set<IClass> clients = myClassDependencyGraph.getClients();
        Set<IPackage> packageClients = myPackageDependencyGraph.getClients();
        Graphics g = graphPane.getGraphics();
        Font font = new Font(Font.MONOSPACED, Font.ITALIC, 15);
        g.setFont(font);
        int letterWidth = g.getFontMetrics().getWidths()['w'];
        int letterHeight = g.getFontMetrics().getHeight();
        int x = 10;
        int y = 10;
        int clHeight = letterHeight + 4;
        int maxHeight = 10;
        for (IPackage pckg : packageClients) {
            int maxClWidth = 0;
            for (IClass cl : pckg.getClasses()) {
                if (clients.contains(cl)) {
                    String clName = cl.getName();
                    maxClWidth = Math.max(letterWidth * clName.length() + 10, maxClWidth);
                }
            }
            String name = pckg.getName();
            int number = pckg.getClasses().size();
            int width = 10 + Math.max(maxClWidth, name.length() * letterWidth) + 10;
            if (x > 800 - width) {
                x = 10;
                y += maxHeight + 10;
                maxHeight = 10;
            }
            int height = (clHeight + 10) * number + letterHeight + 4;
            SuperVertex pv = new SuperVertex(new Point(x, y), name);
            pv.setLetterWidth(letterWidth);
            pv.setLetterHeight(letterHeight);
            pv.setFont(font);
            pv.setHeight(height);
            pv.setWidth(width);
            gr.addVertex(pv);
            packageNameToVertex.put(name, pv);
            int clx = 10 + x;
            int cly = letterHeight + 4 + y;
            for (IClass cl : pckg.getClasses()) {
                if (clients.contains(cl)) {
                    String clName = cl.getName();
                    int clWidth = letterWidth * clName.length() + 10;
                    if (clx > x + width - clWidth) {
                        clx = x + 10;
                        cly += clHeight + 10;
                    }
                    SimpleVertex v = new SimpleVertex(new Point(clx, cly), clName, pv);
                    v.setLetterWidth(letterWidth);
                    v.setLetterHeight(letterHeight);
                    v.setFont(font);
                    pv.addSimpleVertex(v);
                    v.setWidth(clWidth);
                    v.setHeight(clHeight);
                    gr.addVertex(v);
                    classNameToVertex.put(cl.getFullName(), v);
                    clx += clWidth + 10;
                }
            }
            if (cly + clHeight + 10 - y < height) {
                int newHeight = cly + clHeight + 10 - y;
                pv.setHeight(newHeight);
                height = newHeight;
            }
            maxHeight = Math.max(maxHeight, height);
            x += width + 10;
        }
    }

    private void buildArcs(Map<String, SimpleVertex> classNameToVertex, IGraphModel gr) {
        IArcFilter filter = Arc.getFilter();
        if (filter == null) {
            Arc.setFilter(new ArcFilter());
        }
        for (IClass cl : myClassDependencyGraph.getClients()) {
            String clientName = cl.getFullName();
            SimpleVertex clientSimpleVertex = classNameToVertex.get(clientName);
            for (IClassDependency cld : cl.getClassDependencies()) {
                String serverName = cld.getServer().getFullName();
                SimpleVertex serverSimpleVertex;
                if (classNameToVertex.containsKey(serverName)) {
                    serverSimpleVertex = classNameToVertex.get(serverName);
                    gr.addArc(new Arc(clientSimpleVertex, serverSimpleVertex, cld.subdependenciesToStringList()));
                }

            }
        }
    }
}





